@@ -1,21 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2017 Intel Corporation
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+
+
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef __RML_job_automaton_H
@@ -87,8 +87,8 @@ public:
 
     //! Transition 1-->ptr
     /** Should only be called by owner. */
-    void set_and_release( rml::job& job ) {
-        intptr_t value = reinterpret_cast<intptr_t>(&job);
+    void set_and_release( rml::job* job ) {
+        intptr_t value = reinterpret_cast<intptr_t>(job);
         __TBB_ASSERT( (value&1)==0, "job misaligned" );
         __TBB_ASSERT( value!=0, "null job" );
         __TBB_ASSERT( my_job==1, "already set, or not marked busy?" );
@@ -122,7 +122,7 @@ public:
     }
 
     /** Called by non-owner to wait for transition to ptr. */
-    rml::job& wait_for_job() const {
+    rml::job* wait_for_job() const {
         intptr_t snapshot;
         for(;;) {
             snapshot = my_job;
@@ -130,7 +130,7 @@ public:
             __TBB_Yield();
         }
         __TBB_ASSERT( snapshot!=-1, "wait on plugged job_automaton" );
-        return *reinterpret_cast<rml::job*>(snapshot&~1);
+        return reinterpret_cast<rml::job*>(snapshot&~1);
     }
 };
 
