@@ -1,21 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2017 Intel Corporation
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+
+
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include <tbb/tbb_config.h>
@@ -41,8 +41,8 @@ tbb::atomic<int> n_available_hw_threads;
 class MyClient: public ClientBase<tbb::internal::rml::tbb_client> {
     tbb::atomic<int> counter;
     tbb::atomic<int> gate;
-    /*override*/void process( job& j ) {
-        do_process(j);
+    void process( job& j ) __TBB_override {
+        do_process(&j);
         //wait until the gate is open.
         while( gate==0 )
             Harness::Sleep(1);
@@ -128,7 +128,7 @@ void FireUpJobs( MyServer& server, MyClient& client, int n_thread, int n_extra,
                 for( int k=0; k<n_thread; ++k ) 
                     if( client.job_array[k].processing_count!=0 ) 
                         ++n;
-                    if( n>=expected ) break;
+                if( n>=expected ) break;
                 server.yield();
             }
 #if RML_USE_WCRM
@@ -194,7 +194,7 @@ void Initialize()
 
 int TestMain () {
     VerifyInitialization<MyFactory,MyClient>( MaxThread );
-    if ( default_concurrency<1 ) {
+    if ( server_concurrency<1 ) {
          REPORT("The test is not intended to run on 1 thread\n");
          return Harness::Skipped;
     }
